from typing import Dict, Optional

from pydantic import BaseModel


class ThemeManifest(BaseModel):
    name: str
    slug: str
    version: str
    documentation_url: Optional[str]
    support_url: Optional[str]


class Theme(ThemeManifest):
    id: int
    uuid: str
    isDevelopment: bool
    isPublished: bool


class LocalAsset(BaseModel):
    key: str
    value: str
    contentType: str
    checksum: str


class Asset(BaseModel):
    key: str
    contentType: str
    checksum: str
    value: Optional[str]

    id: int
    attachment: Optional[str]
    size: int
    updatedAt: str
    createdAt: str


AssetMap = Dict[str, Asset]



import typer

from riseact import ui
from riseact.topics.settings.services import settings_store_ngrok_token

app = typer.Typer()


@app.command()
def token(token: str):
    settings_store_ngrok_token(token=str(token))
    ui.success("ngrok token stored")



import base64
import hashlib
import random
import string
import urllib.parse
import webbrowser
from http.server import BaseHTTPRequestHandler, HTTPServer
from typing import Optional

import requests

from riseact import SETTINGS, ui
from riseact.topics.auth.data import (
    AccessToken,
    OAuthParams,
    OAuthRefreshTokenPayload,
    OAuthTokenPayload,
)
from riseact.topics.settings.services import settings_store_credentials


def auth_generate_oauth_url() -> tuple[str, str]:
    code_challenge, code_verifier = auth_generate_pkce_codes()

    base_url = f"{SETTINGS.accounts_host}/oauth/authorize/?"

    params: OAuthParams = {
        "response_type": "code",
        "code_challenge_method": "S256",
        "code_challenge": code_challenge,
        "client_id": SETTINGS.client_id,
        "redirect_uri": SETTINGS.redirect_uri,
    }

    url = base_url + urllib.parse.urlencode(params)

    return url, code_verifier


def auth_generate_pkce_codes() -> tuple[str, str]:
    # stringa random
    code_verifier = "".join(
        random.choice(string.ascii_uppercase + string.digits)
        for _ in range(random.randint(43, 128))
    )

    # encode string in utf-8
    code_verifier = code_verifier.encode("utf-8")

    # base64 della stringa
    code_verifier = base64.b64encode(code_verifier)

    # hash sha256 della stringa in base64
    code_challenge = hashlib.sha256(code_verifier).digest()

    # urlencode dell'hash sha256
    code_challenge = (
        base64.urlsafe_b64encode(code_challenge).decode("utf-8").replace("=", "")
    )

    return code_challenge, code_verifier.decode("utf-8")


def auth_wait_for_auth_code() -> Optional[str]:
    auth_code = None

    class OAuthCodeRequestHandler(BaseHTTPRequestHandler):
        def do_GET(self) -> None:
            o = urllib.parse.urlparse(self.path)
            params = urllib.parse.parse_qs(o.query)
            self.send_response(200)
            self.send_header("Content-type", "text/html")
            self.end_headers()
            self.wfile.write(b"You can now close this window")

            if params.get("code"):
                self.server.query_params = params  # type: ignore

        def log_message(self, *args, **kwargs):
            return

    server = HTTPServer(("", 55443), OAuthCodeRequestHandler)
    server.handle_request()

    if "code" in server.query_params:  # type: ignore
        auth_code = server.query_params["code"][0]  # type: ignore

    return auth_code


def auth_get_access_token(auth_code: str, code_verifier: str) -> Optional[AccessToken]:
    base_url = f"{SETTINGS.accounts_host}/oauth/token/"

    payload: OAuthTokenPayload = {
        "client_id": SETTINGS.client_id,
        "code_verifier": code_verifier,
        "code": auth_code,
        "redirect_uri": SETTINGS.redirect_uri,
        "grant_type": "authorization_code",
    }

    response = requests.post(base_url, data=payload)

    try:
        response.raise_for_status()
        data = response.json()
    except Exception:  # FIX ME
        return None  # FIX ME

    return AccessToken(
        access_token=data["access_token"],
        refresh_token=data["refresh_token"],
    )


def auth_refresh_access_token(refresh_token: str) -> Optional[AccessToken]:
    base_url = f"{SETTINGS.accounts_host}/oauth/token/"
    payload: OAuthRefreshTokenPayload = {
        "client_id": SETTINGS.client_id,
        "refresh_token": refresh_token,
        "redirect_uri": SETTINGS.redirect_uri,
        "grant_type": "refresh_token",
    }

    response = requests.post(base_url, data=payload)

    try:
        response.raise_for_status()
        data = response.json()
    except Exception:  # FIX ME
        return None  # FIX ME

    return AccessToken(
        access_token=data["access_token"],
        refresh_token=data["refresh_token"],
    )


def auth_login():
    with ui.console.status("[magenta]Opening browser tab...") as status:
        auth_url, code_verifier = auth_generate_oauth_url()
        webbrowser.open((auth_url))

        status.update(status="[magenta]Waiting for OAuth code from browser...")
        auth_code = auth_wait_for_auth_code()
        if auth_code is None:
            ui.console.print("[bold red]Authentication failed.")
            return

        status.update(status="[magenta]Obtaining access token...")
        oauth_response = auth_get_access_token(auth_code, code_verifier)
        if not oauth_response:
            ui.console.print("[bold red]Authentication failed.")
            return

        status.update(status="[magenta]Storing tokens...")
        settings_store_credentials(
            access_token=oauth_response.access_token,
            refresh_token=oauth_response.refresh_token,
        )

    ui.success("Logged in.")



import json
import os
from dataclasses import dataclass
from pathlib import Path
from typing import Optional, Tuple

from riseact.topics.theme.data import ThemeManifest

MANIFEST_FILENAME = "manifest.json"


@dataclass
class ThemeValidationResult:
    error: bool
    message: Optional[str] = None


VALID_FOLDERS = [
    "assets",
    "config",
    "layout",
    "sections",
    "snippets",
    "templates",
]

FOLDERS_WHITELIST = ["node_modules", ".git", ".github"]


def theme_validate_structure(path: str) -> Tuple[bool, Optional[str]]:
    main_layout_path = Path(path) / "layout" / "theme.html"

    if not os.path.exists(main_layout_path):
        return False, "You must provide default layout/theme.html, check you t7po."

    return True, None


def theme_validate_folders(path: str) -> Tuple[bool, Optional[str]]:
    for entry in os.scandir(path):
        if not entry.is_dir():
            continue

        folder_name = os.path.basename(entry.path)

        if folder_name in FOLDERS_WHITELIST:
            continue

        if folder_name not in VALID_FOLDERS:
            valid_names = ", ".join(VALID_FOLDERS)
            return (
                False,
                f"Folder {folder_name} is not a valid directory name. Allowed names: {valid_names}",
            )

    return True, None


def theme_validate_manifest(path: str) -> Tuple[bool, Optional[str]]:
    manifest_path = Path(path) / MANIFEST_FILENAME

    if not os.path.exists(manifest_path):
        return (
            False,
            f"manifest.json file not found in {path}, are you in the right place?",
        )

    try:
        with open(manifest_path, "r") as f:
            manifest_content = f.read()

        manifest_data = json.loads(manifest_content)
    except Exception:
        return False, "manifest.json is not a valid json file, check your commas."

    try:
        ThemeManifest(**manifest_data)
    except Exception as e:
        return False, f"Some manifest.json fields are not valid: {e}"

    return True, None


def theme_validate_path(path: str) -> ThemeValidationResult:
    result = ThemeValidationResult(error=False)

    manifest_valid, manifest_error = theme_validate_manifest(path)

    if not manifest_valid:
        result.error = True
        result.message = manifest_error
        return result

    folders_valid, folders_error = theme_validate_folders(path)

    if not folders_valid:
        result.error = True
        result.message = folders_error
        return result

    structure_valid, structure_error = theme_validate_structure(path)

    if not structure_valid:
        result.error = True
        result.message = structure_error
        return result

    return result



from dataclasses import dataclass
from typing import Literal, TypedDict

from pydantic import BaseModel


class Application(BaseModel):
    id: int
    name: str
    type: str
    organization: int | None
    appUrl: str | None
    installUrl: str | None
    authorName: str | None
    authorHomepageUrl: str | None
    redirectUris: str | None
    clientId: str
    clientSecret: str


class ApplicationData(BaseModel):
    name: str
    type: str
    organization: int | None


class OAuthParams(TypedDict):
    response_type: Literal["code"]
    code_challenge_method: Literal["S256"]
    code_challenge: str
    client_id: str
    redirect_uri: str


@dataclass
class AppTemplate:
    name: str
    default_dev_command: list[str]


class OAuthTokenPayload(TypedDict):
    client_id: str
    code_verifier: str
    code: str
    redirect_uri: str
    grant_type: Literal["authorization_code"]


class OAuthRefreshTokenPayload(TypedDict):
    client_id: str
    refresh_token: str
    redirect_uri: str
    grant_type: Literal["refresh_token"]


APP_TEMPLATES = [AppTemplate("node", ["yarn", "dev"])]



from datetime import datetime

from rich.console import Console
from rich.padding import Padding
from rich.panel import Panel
from rich.progress import (Progress, RenderableColumn, SpinnerColumn,
                           TimeElapsedColumn)
from rich.table import Table

console = Console()


def table() -> Table:
    return Table(
        show_header=False,
        show_lines=True,
        highlight=True,
    )


def progress() -> Progress:
    return Progress(
        SpinnerColumn(),
        *Progress.get_default_columns(),
        TimeElapsedColumn(),
        transient=False,
        console=console,
    )


def app_dev_progress(footer: str) -> Progress:
    return Progress(
        RenderableColumn(
            Panel(
                Padding(footer, 1),
            )
        ),
        expand=True,
    )


def log_asset_change(msg: str):
    log_time_display = datetime.now().strftime("[%X]")
    console.print(f"[log.time]{log_time_display}[/log.time] {msg}")


def success(msg: str):
    console.print(f"[bold green]✔[/bold green] {msg}")



from riseact.client import execute_query
from riseact.topics.theme.data import Asset, LocalAsset, Theme, ThemeManifest


def theme_create(manifest: ThemeManifest, dev: bool) -> Theme:
    data = {
        "name": f"{manifest.name}",
        "slug": manifest.slug,
        "version": manifest.version,
        "isDevelopment": dev,
    }

    response = execute_query(
        """
        mutation ThemeCreate($data: ThemeInput!) {
            themeCreate(data: $data) {
                id
                uuid
                name
                slug
                version
                documentationUrl
                supportUrl
                isPublished
                isDevelopment
            }
        }
        """,
        variable_values={"data": data},
    )

    theme = Theme(**response["themeCreate"])

    return theme


def theme_delete(theme: Theme) -> None:
    execute_query(
        """
        mutation ThemeDelete($uuid: String!) {
            themeDelete(uuid: $uuid) {
                uuid
            }
        }
        """,
        variable_values={"uuid": theme.uuid},
    )


def asset_create(local_asset: LocalAsset, theme: Theme) -> Asset:
    data = {
        "key": local_asset.key,
        "themeId": theme.id,
        "contentType": local_asset.contentType,
        "value": local_asset.value,
        "attachment": None,
    }

    response = execute_query(
        """
        mutation AssetCreate($data: AssetInput!) {
            assetCreate(data: $data) {
                id
                key
                checksum
                contentType
                value
                attachment
                size
                updatedAt
                createdAt
            }
        }""",
        variable_values={"data": data},
    )

    asset = Asset(**response["assetCreate"])

    return asset


def asset_update(asset: Asset, local_asset: LocalAsset, theme: Theme) -> Asset:
    data = {
        "key": local_asset.key,
        "themeId": theme.id,
        "contentType": local_asset.contentType,
        "value": local_asset.value,
        "attachment": None,
    }

    response = execute_query(
        """
        mutation AssetUpdate($id: Int!, $data: AssetInput!) {
            assetUpdate(id: $id, data: $data) {
                id
                key
                checksum
                contentType
                value
                attachment
                size
                updatedAt
                createdAt
            }
        }""",
        variable_values={"id": asset.id, "data": data},
    )

    asset = Asset(**response["assetUpdate"])

    return asset


def asset_delete(asset: Asset) -> None:
    execute_query(
        """
        mutation AssetDelete($id: Int!) {
            assetDelete(id: $id) {
                id
            }
        }
        """,
        variable_values={"id": asset.id},
    )



import json
import os

from riseact import conf
from riseact.topics.settings.data import Settings

SETTINGS_DATA: Settings | None = None


def settings_load() -> Settings:
    global SETTINGS_DATA

    if not SETTINGS_DATA:
        if conf.CLI_ENV == "STAGING":
            env_data = conf.PROD_ENV
        elif conf.CLI_ENV == "DEV":
            env_data = conf.DEV_ENV
        else:
            env_data = conf.PROD_ENV

        if os.path.exists(conf.SETTINGS_PATH):
            with open(conf.SETTINGS_PATH, "r") as f:
                content = f.read()

            stored_data = json.loads(content)
            args = {**stored_data, **env_data}
        else:
            args = {**env_data}

        SETTINGS_DATA = Settings(**args)

    return SETTINGS_DATA



from pydantic import BaseModel


class Partner(BaseModel):
    id: int
    name: str


class Organization(BaseModel):
    id: int
    name: str
    domain: str
    sitefrontUrl: str


class User(BaseModel):
    name: str
    email: str



from typing import Any, Optional

from gql import Client, gql
from gql.transport.exceptions import TransportQueryError
from gql.transport.requests import RequestsHTTPTransport

from riseact import SETTINGS

CLIENT: Client | None = None


def client_init(partner: bool = False) -> Client:
    headers = {}
    if SETTINGS.access_token:
        headers["Authorization"] = f"Bearer {SETTINGS.access_token}"

    if partner:
        uri = f"{SETTINGS.core_host}/partners/graphql/"
    else:
        uri = f"{SETTINGS.core_host}/graphql/"

    transport = RequestsHTTPTransport(
        url=uri,
        headers=headers,
        verify=True,
        retries=3,
    )

    # Create a GraphQL client using the defined transport
    return Client(transport=transport, fetch_schema_from_transport=True)


def execute_partner_query(
    raw_query: str,
    variable_values: Optional[dict[str, Any]] = None,
):
    global CLIENT

    CLIENT = client_init(partner=True)

    try:
        return CLIENT.execute(gql(raw_query), variable_values)
    except TransportQueryError as e:
        raise e


def execute_query(
    raw_query: str,
    variable_values: Optional[dict[str, Any]] = None,
):
    global CLIENT

    CLIENT = client_init()

    try:
        return CLIENT.execute(gql(raw_query), variable_values)
    except TransportQueryError as e:
        raise e



import os
import re
import shutil
import signal
import subprocess
import sys
import unicodedata
from pathlib import Path
from typing import Optional, TypedDict

import git
import inquirer
import toml
from attr import dataclass
from inquirer.questions import is_pathname_valid
from pyngrok import ngrok

from riseact import SETTINGS, ui
from riseact.client import execute_partner_query
from riseact.topics.app.data import Application, ApplicationData
from riseact.topics.settings.services import settings_store_ngrok_token

ngrok.set_auth_token("25aplhPjeiVEeWP43iTvLF24gN0_5Q6A4Kvr5dxKZUCVNKJLW")

MANIFEST_FILE = "riseact.app.toml"

REPO = "https://github.com/riseact/riseact-app-template-node.git"

sys.path.append(os.path.realpath("."))


def slugify(value, allow_unicode=False):
    """
    Convert to ASCII if 'allow_unicode' is False. Convert spaces or repeated
    dashes to single dashes. Remove characters that aren't alphanumerics,
    underscores, or hyphens. Convert to lowercase. Also strip leading and
    trailing whitespace, dashes, and underscores.
    """
    value = str(value)
    if allow_unicode:
        value = unicodedata.normalize("NFKC", value)
    else:
        value = (
            unicodedata.normalize("NFKD", value)
            .encode("ascii", "ignore")
            .decode("ascii")
        )
    value = re.sub(r"[^\w\s-]", "", value.lower())
    return re.sub(r"[-\s]+", "-", value).strip("-_")


class AppManifest(TypedDict):
    name: str
    dev_commands: list[str]
    client_id: Optional[str]


@dataclass
class AppData(TypedDict):
    path: Path
    manifest: AppManifest


def app_init_form() -> Optional[AppData]:
    app_name = inquirer.text(
        "App name",
        validate=lambda _, x: x not in ["", "."],
    )

    app_path = inquirer.text(
        "App location",
        default=f"./{slugify(app_name)}",
        validate=lambda _, x: is_pathname_valid(x),
    )

    app_template = inquirer.list_input(
        "App template",
        choices=[("Node", "node")],
    )

    app_path = Path(app_path)

    try:
        os.mkdir(app_path)
    except FileExistsError:
        pass

    if app_template == "node":
        dev_commands = ["yarn dev"]
    else:
        dev_commands = []

    return AppData(
        path=app_path,
        manifest=AppManifest(
            name=str(app_name),
            dev_commands=dev_commands,
            client_id=None,
        ),
    )


def app_clone_template(data: AppData):
    git.Repo.clone_from(  # type: ignore
        REPO,
        data["path"],
        branch="master",
        multi_options=["--recurse-submodule"],
    )

    Path(data["path"] / ".gitmodules").unlink(missing_ok=True)

    for git_folder in Path(data["path"]).rglob(".git"):
        if git_folder.is_dir:
            try:
                shutil.rmtree(git_folder, ignore_errors=False)
            except Exception:
                Path(git_folder).unlink(missing_ok=True)
        else:
            Path(git_folder).unlink(missing_ok=True)

    # cloned_repo.submodule_update(recursive=True)


def app_generate_toml(data: AppData) -> str:
    content = toml.dumps(data["manifest"])
    with open(data["path"] / MANIFEST_FILE, "w") as f:
        f.write(content)

    return MANIFEST_FILE


def app_read_manifest() -> AppManifest:
    if not os.path.exists(MANIFEST_FILE):
        ui.console.print("The app manifest cannot be found.")
        sys.exit()

    try:
        with open(MANIFEST_FILE, "r") as f:
            manifest_content = f.read()

        manifest_data = toml.loads(manifest_content)
    except Exception:
        ui.console.print("The app manifest cannot be read.")
        sys.exit()

    return AppManifest(**manifest_data)


def app_start_tunnel() -> ngrok.NgrokTunnel:
    ngrok_token = SETTINGS.ngrok_token
    if ngrok_token is None:
        ngrok_token = inquirer.text("Please paste yout ngrok token")
        settings_store_ngrok_token(token=str(ngrok_token))

    ngrok.set_auth_token(ngrok_token)
    http_tunnel: ngrok.NgrokTunnel = ngrok.connect(3000, bind_tls=True)
    # hmr_tunnel: ngrok.NgrokTunnel = ngrok.connect(3030, bind_tls=True)
    # print(hmr_tunnel)
    ngrok.get_ngrok_process()

    return http_tunnel


def app_install_node_template(data: AppData):
    os.chdir(data["path"])
    app_workdir = os.getcwd()
    process = subprocess.Popen(["yarn"])
    process.wait()

    os.chdir("./src/frontend")
    process = subprocess.Popen(["yarn"])
    process.wait()
    process = subprocess.Popen(["yarn", "build"])
    process.wait()

    os.chdir(app_workdir)


def app_create_dev_process(command: list[str]) -> subprocess.Popen:
    dev_process = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )

    def _signal_handler(signum, frame):
        app_stop_dev_processes(dev_process.pid)

    signal.signal(signal.SIGINT, _signal_handler)

    return dev_process


def app_stop_dev_processes(dev_process_pid: int):
    try:
        with ui.console.status("[magenta]Stopping dev process..."):
            os.killpg(os.getpgid(dev_process_pid), signal.SIGTERM)
        ui.success("Dev process stopped")

        with ui.console.status("[magenta]Stopping ngrok tunnel..."):
            ngrok.kill()
        ui.success("Ngrok tunnel stopped")
    except Exception:
        pass


def app_create(
    *,
    name: str,
    organization_id: int,
) -> Application:
    result = execute_partner_query(
        """
        mutation AppCreate($data: AppInput!) {
            appCreate(data: $data) {
                app {
                    id
                    name
                    type
                    organization
                    appUrl
                    installUrl
                    authorName
                    authorHomepageUrl
                    redirectUris
                    clientId
                    clientSecret
                }
            }
        }""",
        variable_values={
            "data": ApplicationData(
                name=name,
                type="PRIVATE",
                organization=organization_id,
            ).dict(),
        },
    )

    return Application(**result["appCreate"]["app"])


def app_update_redirect_uris(
    *,
    app: Application,
    ngrok_redirect_uri: str,
):
    redirect_uris = (app.redirectUris or "").split()
    redirect_uris = [u for u in redirect_uris if "ngrok.io" not in u]
    redirect_uris.append(ngrok_redirect_uri)

    execute_partner_query(
        """
        mutation RedirectUriMutation($id: Int!, $redirectUris: String!) {
            appUpdateRedirectUris(id: $id, redirectUris: $redirectUris) {
                app {
                    id
                }
            }
        }""",
        variable_values={
            "redirectUris": "\n".join(redirect_uris),
            "id": app.id,
        },
    )



from typing import Optional

from pydantic import BaseModel


class StoredSettings(BaseModel):
    access_token: Optional[str] = None
    refresh_token: Optional[str] = None
    ngrok_token: Optional[str] = None


class Settings(StoredSettings):
    redirect_uri: str
    client_id: str
    admin_host: str
    core_host: str
    accounts_host: str



import webbrowser

import inquirer
import typer

from riseact import SETTINGS, ui
from riseact.topics.app.selectors import (
    app_get,
    app_get_by_client_id,
    app_get_config,
    app_search,
)
from riseact.topics.app.services import (
    app_clone_template,
    app_create,
    app_create_dev_process,
    app_generate_toml,
    app_init_form,
    app_install_node_template,
    app_read_manifest,
    app_start_tunnel,
    app_update_redirect_uris,
)
from riseact.topics.info.selectors import organization_search
from riseact.topics.settings.services import settings_store_ngrok_token

app = typer.Typer()


@app.command()
def init():
    # create new app
    data = app_init_form()
    if not data:
        raise typer.Exit()

    app_clone_template(data)
    filename = app_generate_toml(data)
    ui.success(f"Your '{filename}' has been generated.")
    ui.console.print("Installing app")

    with ui.console.status("[magenta]Installing app..."):
        app_install_node_template(data)
        ui.success("Application created")

    dev()


@app.command()
def dev():
    """
    - load app config
    """
    app_config = app_get_config()
    app_manifest = app_read_manifest()

    current_app = None
    if app_config.get("CLIENT_ID", None):
        current_app = app_get_by_client_id(client_id=app_config["CLIENT_ID"])

    if current_app is None:
        ui.console.print(
            "Looks like this is the first time you're running dev for this project."
        )
        ui.console.print("Configure your preferences by answering a few questions.")
        ui.console.print(
            "\nBefore you preview your work, it needs to be associated with an app."
        )

        create_new_app = inquirer.list_input(
            "Create this project as a new app on Riseact?",
            choices=[
                ("Yes, create it as a new app", True),
                ("No, connect it to an existing app", False),
            ],
            default=True,
        )

        if create_new_app:
            app_name = inquirer.text("App name", default=app_manifest["name"])

            partner_organizations = organization_search()
            app_organization_id = inquirer.list_input(
                "Which organization would you like to use to view your project?",
                choices=[(o.name, o.id) for o in partner_organizations],
            )

            with ui.console.status("[magenta]Creating application...") as status:
                current_app = app_create(
                    name=str(app_name), organization_id=int(app_organization_id)
                )
                ui.success("Application created")
        else:
            partner_apps = app_search()
            app_id = inquirer.list_input(
                "Which existing app is this for?",
                choices=[(a.name, a.id) for a in partner_apps],
            )
            app_id = int(app_id)
            current_app = app_get(id=app_id)

            if current_app is None:
                ui.console.print(
                    "There has been an error retrieving this app. Contact support."
                )
                return

        with ui.console.status("[magenta]Writing .env...") as status:
            with open("./.env", "w") as fh:
                fh.write(
                    "\n".join(
                        [
                            f"CLIENT_ID={current_app.clientId}",
                            f"CLIENT_SECRET={current_app.clientSecret}",
                        ]
                    )
                )
            ui.success(".env updated")

    ngrok_tunnel = app_start_tunnel()
    with ui.console.status("[magenta]Updating redirect URIs...") as status:
        app_update_redirect_uris(
            app=current_app,
            ngrok_redirect_uri=f"{ngrok_tunnel.public_url}/oauth/callback",
        )
        ui.success("Redirect URIs updated")

    if not SETTINGS.ngrok_token:
        ui.console.print(
            "To make your local code accessible to your dev organization, you need to use a"
        )
        ui.console.print("Riseact-trusted tunneling service called ngrok.")
        ui.console.print(
            "To sign up and get an auth token: https://dashboard.ngrok.com/get-started/your-authtoken"
        )

        ngrok_token = inquirer.password(
            "Enter your ngrok token",
            validate=_ngrok_answer_validate,
        )

        settings_store_ngrok_token(token=str(ngrok_token))
        ui.success("Ngrok token stored")

    with ui.console.status("[magenta]Starting ngrok tunnel...") as status:
        ui.success("Ngrok tunnel started")
        status.update("[magenta]Starting dev process...")

        dev_process = app_create_dev_process(["yarn", "dev"])
        ui.success("Dev process started")

    with ui.app_dev_progress(
        f"[bold green]✔[/bold green] Preview URL: {ngrok_tunnel.public_url}"
    ) as progress:
        progress.add_task("", total=None)
        webbrowser.open((ngrok_tunnel.public_url))
        for line in iter(dev_process.stdout.readline, ""):  # type: ignore
            print(line.decode())


def _ngrok_answer_validate(answers, current):
    return bool(current)



import typer

from riseact import ui
from riseact.topics.auth.services import auth_login
from riseact.topics.settings.services import settings_delete_credentials

app = typer.Typer()


@app.command()
def login():
    auth_login()


@app.command()
def logout():
    with ui.console.status("[magenta]Logging out..."):
        settings_delete_credentials()

    ui.success("Logged out from Riseact.")



import typer
from rich import print

from riseact import SETTINGS, conf, ui

app = typer.Typer()


@app.command()
def show():
    table = ui.table()

    for key, value in SETTINGS.dict().items():
        table.add_row(key, value)

    ui.console.print(table)


@app.command()
def path():
    print(conf.SETTINGS_PATH)



import os
from pathlib import Path

from typer import get_app_dir

APP_NAME = "riseact"

APP_DIR = get_app_dir(APP_NAME)

SETTINGS_PATH: Path = Path(APP_DIR) / "config.json"

CLI_ENV = os.getenv("RISEACT_CLI_ENV", "PROD").upper()

DEV_ENV = {
    "redirect_uri": "http://localhost:55443",
    "client_id": "VUpZM6CDKimnGgCSkzxLWsHP60DytxfHeRJXgVA2",
    "admin_host": "http://localhost:4500",
    "core_host": "http://core.localhost:8000",
    "accounts_host": "http://accounts.localhost:8000",
}

PROD_ENV = {
    "redirect_uri": "http://localhost:55443",
    "client_id": "oigtjb908t2i3lnkjvgfjdSFGHY43gk90ufsdfsd",
    "admin_host": "https://admin.riseact.org",
    "core_host": "https://core.riseact.org",
    "accounts_host": "https://accounts.riseact.org",
}
